@@ -70,6 +70,7 @@ struct _GeditDocumentPrivate
 	gchar		*last_searched_text;
 	gchar		*last_replace_text;
 	gboolean  	 last_search_was_case_sensitive;
+	gboolean  	 last_search_was_entire_word;
 
 	guint		 auto_save_timeout;
 	gboolean	 last_save_was_manually; 
@@ -688,8 +689,16 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	if (file_size > 0)
 	{
 		gchar *converted_text;
+		gint len = -1;
 
-		converted_text = gedit_utils_convert_to_utf8 (file_contents,
+		if (g_utf8_validate (file_contents, file_size, NULL))
+		{
+			converted_text = file_contents;
+			len = file_size;
+			file_contents = NULL;
+		}
+		else
+			converted_text = gedit_utils_convert_to_utf8 (file_contents,
 							file_size,
 							&doc->priv->encoding);
 
@@ -711,7 +720,7 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 		gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
 		/* Insert text in the buffer */
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
-		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, converted_text, file_size);
+		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, converted_text, len);
 		g_free (converted_text);
 
 		/* We had a newline in the buffer to begin with. (The buffer always contains
@@ -726,7 +735,8 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 		gedit_undo_manager_end_not_undoable_action (doc->priv->undo_manager);
 	}
 
-	g_free (file_contents);
+	if (file_contents != NULL)
+		g_free (file_contents);
 
 	if (gedit_utils_is_uri_read_only (uri))
 	{
@@ -766,7 +776,13 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	{
 		gchar *converted_text;
 
-		converted_text = gedit_utils_convert_to_utf8 (stdin_data,
+		if (g_utf8_validate (stdin_data, -1, NULL))
+		{
+			converted_text = stdin_data;
+			stdin_data = NULL;
+		}
+		else
+			converted_text = gedit_utils_convert_to_utf8 (stdin_data,
 							-1,
 							&doc->priv->encoding);
 
@@ -797,9 +813,14 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 		gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
 
 		gedit_undo_manager_end_not_undoable_action (doc->priv->undo_manager);
+		
 		g_free (converted_text);
 	}
 
+	if (stdin_data != NULL)
+		g_free (stdin_data);
+			
+
 	gedit_document_set_readonly (doc, FALSE);
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), TRUE);
 
@@ -1072,6 +1093,8 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	gint fd;
 	gint retval;
 	GeditSaveEncodingSetting encoding_setting;
+	gboolean res;
+	gboolean add_cr;
 	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
@@ -1243,7 +1266,17 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	}	    
 
 	chars_len = strlen (chars);
-	if (write (fd, chars, chars_len) != chars_len)
+
+	add_cr = (*(chars + chars_len - 1) != '\n');
+
+	/* Save the file content */
+	res = (write (fd, chars, chars_len) == chars_len);
+
+	if (res && add_cr)
+		/* Add \n if needed */
+		res = (write (fd, "\n", 1) == 1);
+
+	if (!res)
 	{
 		gchar *msg;
 
@@ -1708,7 +1741,7 @@ gedit_document_set_last_replace_text (GeditDocument* doc, const gchar *text)
 
 gboolean
 gedit_document_find (GeditDocument* doc, const gchar* str, 
-		gboolean from_cursor, gboolean case_sensitive)
+		gboolean from_cursor, gboolean case_sensitive, gboolean entire_word)
 {
 	GtkTextIter iter;
 	gboolean found = FALSE;
@@ -1753,13 +1786,31 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	else		
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
 
+	found = FALSE;
+	
 	if (*converted_str != '\0')
     	{
       		GtkTextIter match_start, match_end;
 
-          	found = gedit_text_iter_forward_search (&iter, converted_str, search_flags,
-                                        	&match_start, &match_end,
-                                               	NULL);	
+		while (!found)
+		{
+          		found = gedit_text_iter_forward_search (&iter, converted_str, search_flags,
+                        	                	&match_start, &match_end,
+                                	               	NULL);	
+
+			if (found && entire_word)
+			{
+				found = gtk_text_iter_starts_word (&match_start) && 
+					gtk_text_iter_ends_word (&match_end);
+
+				if (!found) 
+					iter = match_end;
+
+			}
+			else
+				break;
+		}
+		
 		if (found)
 		{
 			gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc),
@@ -1774,6 +1825,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 
 		doc->priv->last_searched_text = g_strdup (str);
 		doc->priv->last_search_was_case_sensitive = case_sensitive;
+		doc->priv->last_search_was_entire_word = entire_word;
 	}
 
 	g_free (converted_str);
@@ -1782,7 +1834,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 }
 
 gboolean
-gedit_document_find_again (GeditDocument* doc)
+gedit_document_find_again (GeditDocument* doc, gboolean from_cursor)
 {
 	gchar* last_searched_text;
 	gboolean found;
@@ -1797,8 +1849,12 @@ gedit_document_find_again (GeditDocument* doc)
 	if (last_searched_text == NULL)
 		return FALSE;
 	
-	found = gedit_document_find (doc, last_searched_text, TRUE, 
-				doc->priv->last_search_was_case_sensitive);
+	found = gedit_document_find (doc, 
+				     last_searched_text, 
+				     from_cursor, 
+				     doc->priv->last_search_was_case_sensitive, 
+				     doc->priv->last_search_was_entire_word);
+
 	g_free (last_searched_text);
 
 	return found;
@@ -1893,7 +1949,7 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 
 gboolean
 gedit_document_replace_all (GeditDocument *doc,
-	      	const gchar *find, const gchar *replace, gboolean case_sensitive)
+	      	const gchar *find, const gchar *replace, gboolean case_sensitive, gboolean entire_word)
 {
 	gboolean from_cursor = FALSE;
 	gboolean cont = 0;
@@ -1906,7 +1962,7 @@ gedit_document_replace_all (GeditDocument *doc,
 
 	gedit_document_begin_user_action (doc);
 
-	while (gedit_document_find (doc, find, from_cursor, case_sensitive)) 
+	while (gedit_document_find (doc, find, from_cursor, case_sensitive, entire_word)) 
 	{
 		gedit_document_replace_selected_text (doc, replace);
 		
