@@ -49,6 +49,8 @@
 #include "gedit-encodings.h"
 #include "gedit-debug.h"
 
+#define STDIN_DELAY_MICROSECONDS 100000
+
 /* =================================================== */
 /* Flash */
 
@@ -1548,13 +1550,19 @@ gedit_utils_get_stdin (void)
 {
 	GString * file_contents;
 	gchar *tmp_buf = NULL;
-	struct stat stats;
 	guint buffer_length;
 	GnomeVFSResult	res;
+	fd_set rfds;
+	struct timeval tv;
 	
-	fstat (STDIN_FILENO, &stats);
-	
-	if (stats.st_size  == 0)
+	FD_ZERO (&rfds);
+	FD_SET (0, &rfds);
+
+	/* wait for 1/4 of a second */
+	tv.tv_sec = 0;
+	tv.tv_usec = STDIN_DELAY_MICROSECONDS;
+
+	if (select (1, &rfds, NULL, NULL, &tv) != 1)
 		return NULL;
 
 	tmp_buf = g_new0 (gchar, GEDIT_STDIN_BUFSIZE + 1);
@@ -1637,12 +1645,13 @@ gedit_utils_convert_to_utf8 (const gchar *content, gsize len,
 	GSList *start;
 	const gchar *locale_charset;
 
-	if (g_utf8_validate (content, len, NULL))
-		return g_strdup (content);
+	g_return_val_if_fail (!g_utf8_validate (content, len, NULL), 
+			g_strndup (content, len < 0 ? strlen (content) : len));
 
 	encodings = gedit_prefs_manager_get_encodings ();
 
-	if (g_get_charset (&locale_charset) == FALSE) {
+	if (g_get_charset (&locale_charset) == FALSE) 
+	{
 		const GeditEncoding *locale_encoding;
 
 		/* not using a UTF-8 locale, so try converting
@@ -1657,7 +1666,8 @@ gedit_utils_convert_to_utf8 (const gchar *content, gsize len,
 
 	start = encodings;
 
-	while (encodings != NULL) {
+	while (encodings != NULL) 
+	{
 		GeditEncoding *enc;
 		const gchar *charset;
 		gchar *utf8_content;
@@ -1672,7 +1682,14 @@ gedit_utils_convert_to_utf8 (const gchar *content, gsize len,
 							(content, len, charset);
 
 		if (utf8_content != NULL) {
-			*encoding_used = g_strdup (charset);
+			if (encoding_used != NULL)
+			{
+				if (*encoding_used != NULL)
+					g_free (*encoding_used);
+				
+				*encoding_used = g_strdup (charset);
+			}
+
 			return utf8_content;
 		}
 
